@@ -21,8 +21,8 @@ use App::Multigit::Loop qw(loop);
 use Exporter 'import';
 
 our @EXPORT_OK = qw/
-    mgconfig mg_parent 
-    all_repositories selected_repositories 
+    mgconfig mg_parent
+    all_repositories selected_repositories
     base_branch set_base_branch mg_each
 /;
 
@@ -33,7 +33,7 @@ deal with git subrepositories.
 
 =cut
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 =head1 PACKAGE VARS
 
@@ -171,7 +171,7 @@ sub all_repositories {
 
 =head2 selected_repositories
 
-This returns the repository configuration as determined by 
+This returns the repository configuration as determined by
 L<C<<@SELECTED_REPOS>>|/@SELECTED_REPOS>. Directories that exist in the main
 config (L<all_repositories>) will have their configuration honoured, but unknown
 directories will have default configuration.
@@ -198,7 +198,7 @@ sub selected_repositories {
             $selected_repos->{ $bydir->{$dir}->{url} } = $bydir->{$dir};
         }
         else {
-            my $url = 
+            my $url =
                 try {
                     _sensible_remote_url($dir);
                 }
@@ -217,10 +217,12 @@ sub selected_repositories {
     return $selected_repos;
 }
 
-=head2 each($command)
+=head2 each($command[, $ia_config])
 
 For each configured repository, C<$command> will be run. Each command is run in
-a separate process which C<chdir>s into the repository first.
+a separate process which C<chdir>s into the repository first. Optionally, the
+C<$ia_config> hashref may be provided; this will be passed to
+L<App::Multigit::Repo/run>.
 
 It returns a convergent L<App::Multigit::Future> that represents all tasks. When
 this Future completes, all tasks are complete.
@@ -248,7 +250,7 @@ the C<< $future->get >> until all repositories have reported completion.
 See C<examples/mg-branch> for a simple implementation of this.
 
 The Future can complete with whatever you like, but be aware that C<run> returns
-a hash-shaped list; see the docs for 
+a hash-shaped list; see the docs for
 L<run|App::Multigit::Repo/"run($command, [%data])">. This means it is often
 useful for the very last thing in your subref to be a transformation - something
 that extracts data from the C<%data> hash and turns it into a usefully-shaped
@@ -369,7 +371,7 @@ success and discards the error information.
 In the arrayref form, the C<$command> is passed directly to C<run> in
 L<App::Multigit::Repo|App::Multigit::Repo/"run($command, [%data])">.  The
 Futures returned thus are collated and the list of return values is thus
-collated. 
+collated.
 
 Because L<run|App::Multigit::Repo/"run($command, [%data])"> completes a Future
 with a hash-shaped list, the convergent Future that C<each> returns will be a
@@ -394,9 +396,10 @@ form is used.
 
 sub each {
     my $command = shift;
+    my $ia_config = shift;
     my $repos = selected_repositories;
 
-    my $f = fmap { _run_in_repo($command, $_[0], $repos->{$_[0]}) } 
+    my $f = fmap { _run_in_repo($command, $_[0], $repos->{$_[0]}, $ia_config) }
         foreach => [ keys %$repos ],
         concurrent => $BEHAVIOUR{concurrent_processes},
     ;
@@ -415,7 +418,7 @@ This is the exported name of C<each>
 *mg_each = \&each;
 
 sub _run_in_repo {
-    my ($cmd, $repo, $config) = @_;
+    my ($cmd, $repo, $config, $ia_config) = @_;
 
     return App::Multigit::Future->done( $config->{dir} => "Readonly" )
         if $BEHAVIOUR{skip_readonly} and $config->{readonly};
@@ -424,7 +427,7 @@ sub _run_in_repo {
         App::Multigit::Repo->new(
             name => $repo,
             config => $config
-        )->run($cmd);
+        )->run($cmd, ia_config => $ia_config);
     }
     else {
         App::Multigit::Repo->new(
@@ -434,14 +437,16 @@ sub _run_in_repo {
     }
 }
 
-=head2 init($workdir)
+=head2 mkconfig($workdir)
 
-Scans C<$workdir> for git directories and registers each in C<.mgconfig>
+Scans C<$workdir> for git directories and registers each in C<.mgconfig>. If the
+config file already exists it will be appended to; existing config will be
+preserved where possible.
 
 =cut
 
-sub init {
-    my $workdir = shift;
+sub mkconfig {
+    my $workdir = shift // mg_parent;
     my @dirs = File::Find::Rule
         ->relative
         ->directory
@@ -454,7 +459,7 @@ sub init {
 
     # If it's already inited, we'll keep the config
     %config = try {
-        %{ all_repositories() }
+        %{ all_repositories($workdir) }
     } catch {};
 
     for my $dir (@dirs) {
@@ -468,11 +473,38 @@ sub init {
         or next;
         $config{$url}->{dir} = $dir;
     }
-    
+
     my $config_filename = dir($workdir)->file(mgconfig);
     Config::INI::Writer->write_file(\%config, $config_filename);
 }
 
+=head2 clean_config
+
+Checks the C<.mgconfig> for directories that don't exist and removes the associated repo section.
+
+=cut
+
+sub clean_config {
+    my $config = all_repositories;
+    my $workdir = shift // mg_parent;
+
+    for my $url (keys %$config) {
+        my $conf = $config->{$url};
+        my $dir = dir($conf->{dir});
+
+        if ($dir->is_relative) {
+            $dir = $dir->absolute($workdir);
+        }
+
+        unless (-e $dir) {
+            delete $config->{$url};
+        }
+    }
+
+    my $config_filename = $workdir->file(mgconfig);
+    Config::INI::Writer->write_file($config, $config_filename);
+}
+
 # Fetch either origin URL, or any URL. Dies if none.
 sub _sensible_remote_url {
     my $dir = shift;
