@@ -5,10 +5,11 @@ use IO::Async::Process;
 use Future;
 use Moo;
 use Cwd 'getcwd';
+use Try::Tiny;
 
 use 5.014;
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 =encoding utf8
 
@@ -66,6 +67,10 @@ to the process's STDIN.
 A Future object is returned. When the command finishes, the Future is completed
 with a hash-shaped list identical to the one C<run> accepts.
 
+If an error occurs I<before> running the command (i.e. if IO::Async throws the
+error), it will behave as though an error occurred within the command, and
+C<exitcode> will be set to 255.
+
 =head3 data
 
 C<run> accepts a hash of data. If C<stdout> or C<stderr> are provided here, the
@@ -93,6 +98,17 @@ C<past_stdout> and C<past_stderr> are never used; they are provided for you to
 write any procedure you may require to concatenate new output with old. See
 C<gather>.
 
+=head3 IO::Async::Process
+
+The special key C<ia_config> to the C<%data> hash will be removed from the hash
+and used as configuration for the L<IO::Async::Process> object that powers the
+whole system.
+
+It currently supports the C<no_cd> option, to prevent attempting to C<chdir>
+into the repo's directory.
+
+  $repo->run($subref, ia_config => { no_cd => 1 });
+
 =cut
 
 sub run {
@@ -102,66 +118,65 @@ sub run {
     bless $future, 'App::Multigit::Future';
 
     $data{stdout} //= '';
+    my $ia_config = delete $data{ia_config};
 
     my $ignore_stdout = $App::Multigit::BEHAVIOUR{ignore_stdout};
     my $ignore_stderr = $App::Multigit::BEHAVIOUR{ignore_stderr};
 
-    if (ref $command eq 'CODE') {
-        loop->run_child(
-            code => sub {
-                $command->($self, %data); 0;
-            },
-            setup => [
-                chdir => $self->config->{dir}
-            ],
-            on_finish => sub {
-                my (undef, $exitcode, $stdout, $stderr) = @_;
-                my %details = (
-                    stdout => $ignore_stdout ? '' : $stdout,
-                    stderr => $ignore_stderr ? '' : $stderr, 
-                    exitcode => $exitcode, 
-                    past_stdout => $ignore_stdout ? '' : $data{stdout},
-                    past_stderr => $ignore_stderr ? '' : $data{stderr},
-                );
-
-                if ($exitcode == 0) {
-                    $future->done(%details);
-                }
-                else {
-                    $future->fail("Child process exited with nonzero exit status", 
-                        exit_nonzero => %details);
-                }
-            }
+    my $finish_code = sub {
+        my (undef, $exitcode, $stdout, $stderr) = @_;
+        my %details = (
+            stdout => $ignore_stdout ? '' : $stdout,
+            stderr => $ignore_stderr ? '' : $stderr,
+            exitcode => $exitcode,
+            past_stdout => $ignore_stdout ? '' : $data{stdout},
+            past_stderr => $ignore_stderr ? '' : $data{stderr},
         );
-    }
-    else {
-        loop->run_child(
-            command => $command,
-            setup => [
-                chdir => $self->config->{dir}
-            ],
-            stdin => $data{stdout},
-            on_finish => sub {
-                my (undef, $exitcode, $stdout, $stderr) = @_;
-                my %details = (
-                    stdout => $ignore_stdout ? '' : $stdout,
-                    stderr => $ignore_stderr ? '' : $stderr,
-                    exitcode => $exitcode,
-                    past_stdout => $ignore_stdout ? '' : $data{stdout},
-                    past_stderr => $ignore_stderr ? '' : $data{stderr},
-                );
 
-                if ($exitcode == 0) {
-                    $future->done(%details);
-                }
-                else {
-                    $future->fail(
-                        "Child process exited with nonzero exit status",
-                        exit_nonzero => %details);
-                }
-            }
-        )
+        if ($exitcode == 0) {
+            $future->done(%details);
+        }
+        else {
+            $future->fail(
+                "Child process exited with nonzero exit status",
+                exit_nonzero => %details);
+        }
+    };
+
+    try
+    {
+        my $setup = [];
+        unless($ia_config->{no_cd}) {
+            $setup = [
+                chdir => $self->config->{dir}
+            ];
+        }
+        if (ref $command eq 'CODE') {
+            loop->run_child(
+                code => sub {
+                    $command->($self, %data); 0;
+                },
+                setup => $setup,
+                on_finish => $finish_code,
+            );
+        }
+        else {
+            loop->run_child(
+                command => $command,
+                setup => $setup,
+                stdin => $data{stdout},
+                on_finish => $finish_code,
+            )
+        }
     }
+    catch
+    {
+        # make failures coming from the Async code come out as an error
+        # relating to the repo as they probably are.
+        # rather than crashing the whole program hard.
+        # the common error case is the subdirectory for the module not existing.
+        $finish_code->(undef, 255, '', "Error running\n" . $_);
+    };
     return $future;
 }
 
